#!/bin/bash
# chkconfig: 345 98 02
# description: ZeroTurnaround License Server

cygwin=false
os400=false
darwin=false
case "`uname`" in
  CYGWIN*) cygwin=true;;
  OS400*) os400=true;;
  Darwin*) darwin=true;;
esac

# resolve links - $0 may be a softlink
PRG="$0"

while [ -h "$PRG" ]; do
  ls=`ls -ld "$PRG"`
  link=`expr "$ls" : '.*-> \(.*\)$'`
  if expr "$link" : '/.*' > /dev/null; then
    PRG="$link"
  else
    PRG=`dirname "$PRG"`/"$link"
  fi
done

# Get standard environment variables
PRGDIR=`dirname "$PRG"`
LS_HOME=`cd "$PRGDIR/.." >/dev/null; pwd`
PID_FILE=$LS_HOME/RUNNING_PID

# For Cygwin, ensure paths are in UNIX format before anything is touched
if $cygwin ; then
  [ -n "$LS_HOME" ] &&
    LS_HOME=`cygpath --unix "$LS_HOME"`
fi

if [ "x$LS_HOME" = "x" ]; then
  # get the full path (without any relative bits)
  LS_HOME=`cd $DIRNAME/..; pwd`
fi

cd "$LS_HOME";

# For Cygwin, switch paths to Windows format before running java
if $cygwin; then
  LS_HOME=`cygpath --path --windows "$LS_HOME"`
fi

# Setup the JVM
if [ "x$JAVA" = "x" ]; then
  if [ "x$JAVA_HOME" != "x" ]; then
    JAVA="$JAVA_HOME/bin/java"
  else
    JAVA="java"
  fi
fi

############################ METHOD DEFINITIONS ############################

do_start ()
{
  mkdir -p "$LS_HOME/logs"
  cleanup_pid
  nohup "$JAVA" -classpath "$LS_HOME/lib/license-server.jar" $JAVA_OPTS "$@" com.zeroturnaround.ls.Startup > $LS_HOME/logs/license-server.out 2>&1 < /dev/null &
  echo "Starting License Server. Console output is logged into $LS_HOME/logs/license-server.out"
}

do_run ()
{
  cleanup_pid
  "$JAVA" -classpath "$LS_HOME/lib/license-server.jar" $JAVA_OPTS "$@" com.zeroturnaround.ls.Startup
}

do_stop ()
{
  "$JAVA" -classpath "$LS_HOME/lib/license-server.jar" $JAVA_OPTS "$@" com.zeroturnaround.ls.Shutdown

  if [ "$RESTART" = "true" ]; then
    RESTART=
    do_start $@
  fi
}

do_status ()
{
  "$JAVA" -classpath "$LS_HOME/lib/license-server.jar" $JAVA_OPTS "$@" com.zeroturnaround.ls.Status
}


cleanup_pid ()
{
  if [ -e "$PID_FILE" ]; then

    # Read the old PID from file
    PID=$(cat "$PID_FILE")

    # Check if a process with such PID is still running.
    # will be 0 if such process is not running, >0 otherwise
  
    IS_RUNNING=$(ps aux | grep $PID | grep -v -c grep)

    if [ $IS_RUNNING = '0' ];
    then
      echo "Old process looks dead .. cleaning up RUNNING_PID at \"$PID_FILE\""
      rm -f "$PID_FILE"
    fi

  fi
}


############################################################################

if [ "start" = "$1" ]; then
  shift
  do_start $@
elif [ "run" = "$1" ]; then
  shift
  do_run $@
elif [ "stop" = "$1" ]; then
  shift
  do_stop $@
elif [ "restart" = "$1" ]; then
  shift
  RESTART=true
  do_stop $@
elif [ "status" = "$1" ]; then
  shift
  do_status $@
else
  echo "Usage:  license-server ( commands ... )"
  echo "commands:"
  echo   "run               Start License Server in this terminal"
  echo   "start             Start License Server in background"
  echo   "restart           Restart License Server"
  echo   "stop              Stop License Server"
  echo   "status            Check License Server status"
fi

