$(function () {
  LS.currencySign = {
    EUR: '€',
    USD: '$'
  };

  LS.customValidationMessages = {
    annualDevSalary: {
      tooBig: "Maximum value for yearly salary is 10M.",
      tooSmall: "Minimum value for yearly salary is 1000.",
      stepMismatch: "Yearly salary should be an integer value."
    },
    devCount: {
      tooBig: "Maximum value for number of developers is 10,000.",
      tooSmall: "Minimum value for number of developers is 1.",
      stepMismatch: "Number of developers should be an integer value."
    },
    redeployTimeInMin: {
      tooBig: "Maximum value for build and redeploy time is 60 minutes.",
      tooSmall: "Minimum value for build and redeploy time is 0.1 minutes.",
      stepMismatch: "Build and redeploy time should be defined with precision of 0.1 minutes."
    },
    redeploysPerHour: {
      tooSmall: "Minimum value for application restarts is 1 per hour.",
      stepMismatch: "Application restarts per hour should be an integer value."
    },
    totalRedeployTimeTooBig: "Redeploy time * restarts per hour must be less than 60 minutes."
  };

  //set initial field values
  fillParams(LS.serverRoiParams);

  //calculate values
  updateRoiResults();

  //recalculate on changes
  var roiParams = $("#roiParams");

  roiParams.find("select").on('change', updateRoiResults);
  //onChange event works badly with incremental stepper, use click and keyup for input fields instead
  roiParams.find("#annualDevSalary, #devCount, #redeployTimeInMin, #redeploysPerHour")
      .on('keyup click', onFieldValueChanged);

  //disable any action on fake submit. Should not happen but just in case;
  roiParams.on('submit', function (e) {
    //prevent the form from doing a submit
    e.preventDefault();
    return false;
  });

  $("#exportToPdfButton").click(function () {
    var params = scanParams();
    var result = computeRoi(params);
    var query = LS.routes.exportToPdf + '?' + $.param({
      devCount: params.devCount,
      redeployTimeInMin: (params.redeployTimeInSec / 60).toFixed(1),
      redeploysPerHour: params.redeploysPerHour,
      annualDevSalary: Math.round(params.annualDevSalary).toLocaleString('en-US'),
      currency: params.currency,
      extraDeveloperNeeded: Math.round(result.extraDeveloperNeeded),
      extraDeveloperTotalCost: Math.round(result.extraDeveloperTotalCost).toLocaleString('en-US'),
      numberOfTotalDevsNeeded: Math.round(result.numberOfTotalDevsNeeded),
      JRebelTeamCost: Math.round(result.JRebelTeamCost).toLocaleString('en-US'),
      extraDeveloperNeededPercent: Math.max(0, Math.round(result.extraDeveloperNeededPercent)).toLocaleString('en-US'),
      moneySavedPerYear: Math.max(0, Math.round(result.moneySavedPerYear)).toLocaleString('en-US'),
      paysItselfInDays: Math.ceil(result.paysItselfInDays)
    });
    window.open(query);
  });
});

function resetSavedParams() {
  $.post(LS.routes.resetRoiParams, function () {
    location.reload();
  });
}

function onFieldValueChanged() {
  if (this.id === 'redeploysPerHour' || this.id === 'redeployTimeInMin') {
    checkTotalTimeValidity(this);
  } else {
    this.setCustomValidity('');
  }
  if (this.checkValidity()) {
    updateRoiResults();
  } else {
    setCustomErrorMessage(this);
    $('#submit-button').click();
  }
}

function checkTotalTimeValidity(input) {
  //reset invalidity of both fields in order to only field we are interacting currently will be invalid.
  //otherwise in case of both fields ar marked as invalid we end up with annoying auto focus change
  $("#roiParams").find('#redeploysPerHour, #redeployTimeInMin').each(function (i, v) {
    v.setCustomValidity('');
  });

  if (input.checkValidity()) {
    var commonTime = calculateCommonTime();
    if (commonTime >= 3600) {
      //other cases (like == 0 or invalid formats) should be caught by separate field validators
      input.setCustomValidity(LS.customValidationMessages.totalRedeployTimeTooBig);
    }
  }
}

function calculateCommonTime() {
  var redeployTimeElement = $("#redeployTimeInMin");
  var redeploysPerHourElement = $("#redeploysPerHour");
  var redeployTime = Math.round(parseFloat(redeployTimeElement.val()) * 60);
  var redeploysPerHour = parseFloat(redeploysPerHourElement.val());
  return redeployTime * redeploysPerHour;
}

/**
 * Replaces standard validation error messages with our custom ones if we override it in LS.customValidationMessages table
 * @param element
 */
function setCustomErrorMessage(element) {
  if (!element.validity.customError) {
    var validityError;
    if (element.validity.rangeOverflow) {
      validityError = 'tooBig';
    } else if (element.validity.rangeUnderflow) {
      validityError = 'tooSmall';
    } else if (element.validity.stepMismatch) {
      validityError = 'stepMismatch';
    }

    if (LS.customValidationMessages[element.id]) {
      var customMessage = LS.customValidationMessages[element.id][validityError];
      if (customMessage) {
        element.setCustomValidity(customMessage);
      }
    }
  }
}

function updateRoiResults() {
  //take values
  var params = scanParams();

  //calculate
  var result = computeRoi(params);

  //schedule saving
  scheduleRoiInputsSave(params);

  //set result values
  console.log("ROI Calculated savings: " + JSON.stringify(result));
  $("#minutesSavedPerDayPerDev").text(Math.max(0, Math.round(result.minutesSavedPerDayPerDev)).toLocaleString('en-US'));
  $("#hoursSavedPerYear").text(Math.max(0, Math.round(result.hoursSavedPerYear)).toLocaleString('en-US'));
  $(".currencySign").text(LS.currencySign[params.currency]);
  $("#moneySavedPerYear").text(Math.max(0, Math.round(result.moneySavedPerYear)).toLocaleString('en-US'));
  var paysForItselfInDays = Math.ceil(result.paysItselfInDays);
  if (paysForItselfInDays <= 365) {
    $("#paysItselfInDays").text(paysForItselfInDays);
    if (paysForItselfInDays === 1) {
      $("#paysItselfInDaysText").text("day");
    } else {
      $("#paysItselfInDaysText").text("days");
    }
    $("#paysForItselfBlock").show();
  } else {
    $("#paysForItselfBlock").hide();
  }

  var actualData = computeActualRoi(params);

  var hoursSavedInLastPeriodNumber = Math.ceil(actualData.hoursSavedInLastPeriodNumber);
  if (hoursSavedInLastPeriodNumber === 1) {
    $("#hoursSavedInLastPeriodText").text("hour");
  }
  else {
    $("#hoursSavedInLastPeriodText").text("hours");
  }
  $('#hoursSavedInLastPeriodNumber').text(hoursSavedInLastPeriodNumber.toLocaleString('en-US'));
  console.log("ROI Actual savings: " + JSON.stringify(actualData));
  $('#selectedPeriodText').text(actualData.selectedPeriodText);
  $('#moneySavedInLastPeriod').text(Math.max(0, Math.round(actualData.moneySavedInLastPeriod)).toLocaleString('en-US'));
  $('#absoluteTimeSavingsYearly').text(Math.max(0, Math.round(actualData.absoluteTimeSavingsYearly)).toLocaleString('en-US'));
  $('#actualMinutesSavedPerDayPerDev').text(Math.max(0, Math.round(actualData.actualMinutesSavedPerDayPerDev)).toLocaleString('en-US'));
  $('#actualMoneySavedPerYear').text(Math.max(0, Math.round(actualData.actualMoneySavedPerYear)).toLocaleString('en-US'));

  var actualPaysForItselfInDays = Math.ceil(actualData.paysItselfInDays);
  $("#actualPaidItselfInDays").text(actualPaysForItselfInDays);
  if (actualPaysForItselfInDays === 1) {
    $("#actualPaidItselfInDaysText").text("day");
  }
  else {
    $("#actualPaidItselfInDaysText").text("days");
  }
}

var autoSaveTimeoutId;

function scheduleRoiInputsSave(params) {
  clearTimeout(autoSaveTimeoutId);
  autoSaveTimeoutId = setTimeout(function () {
    if (params && changesDetected(params)) {
      LS.serverRoiParams = params;
      $.post(LS.routes.saveRoiParams, params, function () {
      });
    }
  }, 1000);
}

function scanParams() {
  return {
    currency: $("#currencySelect").val(),
    annualDevSalary: parseFloat($("#annualDevSalary").val()),
    devCount: parseFloat($("#devCount").val()),
    redeployTimeInSec: Math.round(parseFloat($("#redeployTimeInMin").val()) * 60),
    redeploysPerHour: parseFloat($("#redeploysPerHour").val()),
    selectedTimeWindow: $('#periodSelect').val()
  };
}

function fillParams(params) {
  $("#currencySelect").val(params.currency);
  $("#annualDevSalary").val(params.annualDevSalary);
  $("#devCount").val(params.devCount);
  $("#redeployTimeInMin").val((params.redeployTimeInSec / 60).toFixed(1));
  $("#redeploysPerHour").val(params.redeploysPerHour);
}

function changesDetected(params) {
  var lastParams = LS.serverRoiParams;
  return params.currency !== lastParams.currency ||
      params.annualDevSalary !== lastParams.annualDevSalary ||
      params.devCount !== lastParams.devCount ||
      params.redeploysPerHour !== lastParams.redeploysPerHour ||
      params.redeployTimeInSec !== lastParams.redeployTimeInSec;
}

function computeRoi(params) {
  var result = {};

  var redeployTimeInHours = params.redeployTimeInSec / 3600;
  //part of each hour wasted on redeploys. Should be in range (0..1)
  var partOfEachHourWasted = Math.min(redeployTimeInHours * params.redeploysPerHour, 0.99);
  var hoursPerDayWasted = partOfEachHourWasted * LS.HOURS_PER_WORKDAY;
  result.minutesSavedPerDayPerDev = hoursPerDayWasted * 60;
  result.hoursSavedPerYear = hoursPerDayWasted * LS.WORKDAYS_PER_YEAR * params.devCount;

  var extraDeveloperNeededPart = partOfEachHourWasted / (1 - partOfEachHourWasted);
  result.extraDeveloperNeeded = extraDeveloperNeededPart * params.devCount;
  result.extraDeveloperTotalCost = result.extraDeveloperNeeded * params.annualDevSalary;
  result.numberOfTotalDevsNeeded = result.extraDeveloperNeeded + params.devCount;
  result.JRebelTeamCost = LS.jrebelPrices[params.currency] * params.devCount;
  result.extraDeveloperNeededPercent = extraDeveloperNeededPart * 100;

  var savingsPerDevPerDay = extraDeveloperNeededPart * params.annualDevSalary / 365;
  result.paysItselfInDays = LS.jrebelPrices[params.currency] / savingsPerDevPerDay;
  result.moneySavedPerYear = (params.annualDevSalary * extraDeveloperNeededPart - LS.jrebelPrices[params.currency]) * params.devCount;
  return result;
}

function computeActualRoi(params) {
  var result = {};

  var selectedTimeWindow = params.selectedTimeWindow;
  var hoursSavedInLastPeriodNumber = LS.actualSavedTimeHours[selectedTimeWindow];
  var averageDeveloperCountInLastPeriod = LS.averageDeveloperCount[selectedTimeWindow];
  result.hoursSavedInLastPeriodNumber = hoursSavedInLastPeriodNumber;

  var periodLength;
  if (selectedTimeWindow === 'all') {
    result.selectedPeriodText = 'in total';
    periodLength = LS.totalUptimeDays;
  }
  else {
    var timeString;
    if (selectedTimeWindow === 'week') {
      timeString = '7 days';
      periodLength = 7;
    }
    else if (selectedTimeWindow === 'month') {
      timeString = '30 days';
      periodLength = 30;
    }
    else {
      timeString = '12 months';
      periodLength = LS.DAYS_PER_YEAR;
    }
    result.selectedPeriodText = 'over the last ' + timeString;
  }

  var timeSavedDailyPerDeveloper = getTimeSavedDailyPerDeveloper(hoursSavedInLastPeriodNumber, periodLength, averageDeveloperCountInLastPeriod, LS);
  var extraDevelopersNeeded = timeSavedDailyPerDeveloper / (LS.HOURS_PER_WORKDAY - timeSavedDailyPerDeveloper);
  var jrebelPrice = LS.jrebelPrices[params.currency];

  result.moneySavedInLastPeriod = (averageDeveloperCountInLastPeriod * extraDevelopersNeeded * params.annualDevSalary * periodLength) / LS.DAYS_PER_YEAR;
  result.absoluteTimeSavingsYearly = (timeSavedDailyPerDeveloper * LS.WORKDAYS_PER_YEAR * averageDeveloperCountInLastPeriod);
  result.actualMinutesSavedPerDayPerDev = timeSavedDailyPerDeveloper * 60;
  result.actualMoneySavedPerYear = (params.annualDevSalary * averageDeveloperCountInLastPeriod * timeSavedDailyPerDeveloper) / (LS.HOURS_PER_WORKDAY - timeSavedDailyPerDeveloper) - LS.jrebelSeatCount * jrebelPrice;
  result.paysItselfInDays = jrebelPrice * LS.DAYS_PER_YEAR / (extraDevelopersNeeded * params.annualDevSalary);

  return result;
}

function getTimeSavedDailyPerDeveloper(hoursSavedLastPeriod, periodLength, averageDeveloperCountLastPeriod, LS) {
  var timeSaved = (averageDeveloperCountLastPeriod === 0) ? 0 : hoursSavedLastPeriod / (periodLength * averageDeveloperCountLastPeriod * LS.WORKDAYS_PER_YEAR / LS.DAYS_PER_YEAR);
  return Math.min(timeSaved, LS.HOURS_PER_WORKDAY - 0.01);
}
